<?php

/**
  * PHP wrapper class for the GoogleMaps API.
  *
  * @package html
  * @author Norman Heino
  * @version $Id: GoogleMap.php 887 2007-03-25 08:24:00Z nheino $
  */
class OntoWiki_Html_GoogleMap {
	
	const maps_uri = 'http://maps.google.com/maps?file=api&v=2';
	const geo_uri = 'http://maps.google.com/maps/geo?';
	
	private $api_key;
	
	// array of JavaScript GControl implementing class names
	private $controls;
	
	// associative array of the form array(event => callback, ...)
	private $listeners;
	
	// an array of 2-dim arrays
	private $markers;
	private $activeMarker;
	private $center;
	private $zoom;
	private $unbound;
	private $maxZoom;
	
	private $latSum;
	private $lonSum;
	
	private $maxNorth = -90.0;
	private $maxSouth = 90.0;
	private $maxWest  = 180.0;
	private $maxEast  = -180.0;
	
	// denotes whether this map contains any markers
	private $hasMarkers;
	
	/**
	  * Constructs an OWGoogleMap object.
	  *
	  * @param string $key The GoogleMaps API key for the URI this object will be used at.
	  *        Get you key <a href="http://www.google.com/apis/maps/signup.html">here</a>.
	  */
	public function __construct($key, $maxZoom = 10) {
		$this->api_key = $key;
		$this->controls = array();
		$this->listeners = array();
		$this->markers = array();
		$this->activeMarkers = array();
		$this->minMarkers = array();
		
		$this->latSum = 0;
		$this->lonSum = 0;
		
		$this->maxZoom = $maxZoom;
		
		$this->hasMarkers = false;
	}
	
	/**
	  * Returns the map object as a string containing html and JavaScrip.
	  */
	public function __toString() {
		// import api script
		$ret = '<script type="text/javascript" src="' . self::maps_uri . '&amp;v=2&amp;key=' . $this->api_key . '"></script>' . PHP_EOL;
		
		// the div that holds the map
		// TODO: remove size styles
		$ret .= '<div id="map"></div>' . PHP_EOL;
		
		$ret .= '<script type="text/javascript">' . PHP_EOL . '//<![CDATA[' . PHP_EOL;
		$ret .= 'function loadGMap2() {' . PHP_EOL;
		$ret .= '  var map = new GMap2(document.getElementById("map"));' . PHP_EOL;
		
		// add controls to the map
		foreach ($this->controls as $controlClass) {
			$ret .= '  map.addControl(new ' . $controlClass . '());' . PHP_EOL;
		}
		
		// add event listeners to the map
		foreach ($this->listeners as $event => $callback) {
			$ret .= PHP_EOL . '  GEvent.addListener(map, "' . $event . '", ' . $callback . ');' . PHP_EOL;
		}
		
		// create the marker batch
		if ($this->hasMarkers) {
			// a call of the center method is needed before adding markers
			$ret .= '  map.setCenter(new GLatLng(0, 0), 0);' . PHP_EOL;

			// the bounds object to calculate the optimal zoom level
			$ret .= '  var bounds = new GLatLngBounds();' . PHP_EOL;
			
			$ret .= PHP_EOL . '  var batch = [];' . PHP_EOL;
			
			// add markers to the map
			foreach ($this->markers as $marker) {
				$ret .= '  batch.push(new GMarker(new GLatLng(' . $marker['lat'] . ', ' . $marker['lon'] . ')));' . PHP_EOL;
				$ret .= '  bounds.extend(new GLatLng(' . $marker['lat'] . ', ' . $marker['lon'] . '));' . PHP_EOL;
			}

			// add clickable markers to the map
			if (!empty($this->activeMarkers)) {
				$ret .= PHP_EOL . '  function createMarker(point, html) {' . PHP_EOL;
				$ret .= '    var marker = new GMarker(point)' . PHP_EOL;
				$ret .= '      GEvent.addListener(marker, "click", function() {' . PHP_EOL;
				$ret .= '        marker.openInfoWindowHtml(html);' . PHP_EOL;
				$ret .= '      });' . PHP_EOL;
				$ret .= '    return marker;' . PHP_EOL;
				$ret .= '  }' . PHP_EOL . PHP_EOL;

				foreach ($this->activeMarkers as $activeMarker) {
					$ret .= '  batch.push(createMarker(new GLatLng(' . 
						$activeMarker['lat'] . ', ' . $activeMarker['lon'] . '), \'' . $activeMarker['html'] . '\'));' . PHP_EOL;
					$ret .= '  bounds.extend(	new GLatLng(' . $activeMarker['lat'] . ', ' . $activeMarker['lon'] . '));' . PHP_EOL;
				}
			}
			
			$ret .= PHP_EOL . '  var mgr = new GMarkerManager(map);' . PHP_EOL;
			$ret .= '  mgr.addMarkers(batch, 0);' . PHP_EOL;
			$ret .= '  mgr.refresh();' . PHP_EOL;
			
			// set the map's center to the mean of all markers
			$center = $this->getMarkerCenter();
			$ret .= '  var optZoom = map.getBoundsZoomLevel(bounds);' . PHP_EOL;
			$ret .= '  map.setCenter(new GLatLng(' . $center['lat'] . ', ' . 
													 $center['lon'] . '), ' . 
													 "(optZoom < {$this->maxZoom} ? optZoom : {$this->maxZoom}));" . PHP_EOL;
		} else {
			if (isset($this->center)) {
				$ret .= '  map.setCenter(new GLatLng(' . 
						$this->center['lat'] . ', ' . $this->center['lon'] . '), ' . 
						($this->zoom ? $this->zoom : 10) . ');' . PHP_EOL;
			} else {
				$ret .= '  map.setCenter(new GLatLng(51.335540, 12.385190), 10);' . PHP_EOL;
			}
		}
		
		$ret .= '}' . PHP_EOL;
		$ret .= '//]]>' . PHP_EOL . '</script>' . PHP_EOL;
		
		return $ret;
	}
	
	/**
	  * Centers the map at the given coordinates. Subsequent calls of setCenter will 
	  * replace values set by previous calls.
	  *
	  * @param array $coords An associative array containing lat and lon keys and values.
	  * @param integer $zoom The map's zoom level. Possible values as of GoogleMaps
	  *        API version 2 are in the range of 0 .. 20.
	  */
	public function setCenter($coords, $zoom = null) {
		if ($coords['lat'] < -90 || $coords['lat'] > 90 || $coords['lon'] < -180 || $coords['lon'] > 180) {
			$this->unbound = true;
		} else {
			$this->unbound = false;
		}
		$this->center = $coords;
		$this->zoom = $zoom;
	}
	
	/**
	  * Sets the zoom level for the map.
	  *
	  * @param integer $zoom The map's zoom level. Possible values as of GoogleMaps
	  *        API version 2 are in the range of 0 .. 20.
	  */
	public function setZoom($zoom) {
		$this->zoom = $zoom;
	}
	
	/**
	  * Adds the specified GoogleMaps API control to the map.
	  *
	  * @param string $controlClassName The name of the GoogleMaps API control class.
	  *        Availabe options are:
	  *        <ul>
	  *          <li>GLargeMapControl</li>
	  *          <li>GSmallMapControl</li>
	  *          <li>GSmallZoomControl</li>
	  *          <li>GScaleControl</li>
	  *          <li>GMapTypeControl</li>
	  *          <li>GOverviewMapControl</li>
	  *        </ul>
	  */
	public function addControl($controlClassName) {
		$this->controls[] = $controlClassName;
	}
	
	/**
	  * Adds an event listener to the map. Only map events can be listened to
	  * at the moment.
	  *
	  * @param string $event The event to be listened to.
	  * @param string $function The callback function to be called.
	  */
	public function addListener($event, $function) {
		$this->listeners[$event] = $function;
	}
	
	/**
	  * Adds a marker to the map. An optional html string can be passed 
	  * which creates a clickable marker.
	  *
	  * @param array $coords An associative array containing lat and lon keys
	  *        and values. 
	  */
	public function addMarker($coords, $html = null) {
		$this->hasMarkers = true;
		
		if (!$html) {
			$this->markers[] = $coords;
		} else {
			$this->activeMarkers[] = array_merge($coords, array('html' => $html));
		}
		
		$this->latSum += $coords['lat'];
		$this->lonSum += $coords['lon'];

		$this->maxNorth = ($coords['lat'] > $this->maxNorth) ? $coords['lat'] : $this->maxNorth;
		$this->maxSouth = ($coords['lat'] < $this->maxSouth) ? $coords['lat'] : $this->maxSouth;
		
		$this->maxEast = ($coords['lon'] > $this->maxEast) ? $coords['lon'] : $this->maxEast;
		$this->maxWest = ($coords['lon'] < $this->maxWest) ? $coords['lon'] : $this->maxWest;
	}
	
	/**
	  * Queries the Google geocoder server via HTTP and returns latitude and longitude
	  * for a given address. For the request it makes use of PHP's cURL library.
	  *
	  * @param string $address The adress whose geodata are requested.
	  * @return An associativ earray of two values <code>array(lat, lon)</code> or a 
	  * <a href="http://www.google.com/apis/maps/documentation/reference.html#GGeoStatusCode">
	  * GGeoStatusCode</a> if an error occured or false if the service was not reachable.
	  */
	public function getLatLng($address) {
		$ch = curl_init();
		curl_setopt($ch, CURLOPT_URL, self::geo_uri . 'q=' . urlencode($address) . '&output=csv&key=' . $this->api_key);
		curl_setopt($ch, CURLOPT_HEADER, 0);
		curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);

		if ($resp = curl_exec($ch)) {
			curl_close($ch);
			// we requested the response to be returned as comma separated values, 
			// thus we can use explode to create an array
			$resp = explode(',', $resp);
			// if no errors occurred and the address was successfully parsed, return lat and lon 
			if ($resp[0] == '200') {
				return array('lat' => $resp[2], 'lon' => $resp[3]);
			}
			// else return Google's status code
			return $resp[0];
		}
		curl_close($ch);
		return false;
	}
	
	/*
	 * Calculates the mean of all markes that can be used as a map center point.
	 */
	private function getMarkerCenter() {
		$numMarkers = count($this->markers) + count($this->activeMarkers);
		return array('lat' => $this->latSum / $numMarkers, 'lon' => $this->lonSum / $numMarkers);
	}
}
?>