<?php
/**
  * Represents an implementation of an HTML header. It can include .js and .css
  * files upon request. This class needs to be used with php's output buffering
  * functionality as the html header needs to precede any other content sent.
  *
  * @package html
  * @author Norman Heino
  * @version $Id: Header.php 887 2007-03-25 08:24:00Z nheino $
  */
class OntoWiki_Html_Header {
	
	protected $type;
	protected $title;
	protected $base;
	protected $lang;
	protected $enc;
	protected $css_includes = array();
	protected $js_includes = array();
	protected $scripts = array();
	protected $strings = array();
	protected $onloadListener;
	
	/**
	  * Constructs a OWHtmlHeader based on the values provided or default values.
	  *
	  * @param $type The type of HTML header the constructed object should have.
	  *        The following values are possible:<br />
	  *        &quot;strict&quot; &ndash HTML 4.01 Strict<br />
	  *        &quot;loose&quot; &ndash HTML 4.01 Transitional<br />
	  *        &quot;xhtml1-frameset&quot; &ndash XHTML 1.0 Frameset<br />
	  *        &quot;xhtml1-strict&quot; &ndash; XHTML 1.0 Strict
	  *        &quot;xhtml1-transitional&quot; & ndash; XHTML 1.0 Transitional<br />
	  *        &quot;xhtml11&quot; &ndash; XHTML 1.1. The default value ist
	  *        &quot;xhtml1-strict&quot;.
	  * @param $title The HTML page's title. The default value is ''.
	  * @param $base The base adress for the HTML page. The default value is ''.
	  * @param $lang The language that is to be set as a meta http-equiv (default: en).
	  * @param $enc The character encoding for hte HTML file (default: 'utf-8').
	  */
	public function __construct($type = 'xhtml1-strict', $title = '', $base = '', $lang = 'en', $enc = 'utf-8') {
		$this->type = $type;
		$this->title = $title;
		$this->lang = $lang;
		$this->enc = $enc;
		
		// if (($base != '') && (substr($base, -1, 1) != '/'))
		// 	$this->base = $base . '/';
		// else
		// 	$this->base = $base;
	}
	
	/**
	  * Destructs the associated header object and unsets all instance variables.
	  */
	// public function __desctruct() {
	// 	unset($type);
	// 	unset($title);
	// 	unset($base);
	// 	unset($lang);
	// 	unset($enc);
	// 	unset($css_includes);
	// 	unset($js_includes);
	// 	unset($scripts);
	// 	unset($strings);
	// }
	
	/**
	  * Returns the Header object as a string so that it can be written to a file or sent
	  * to the browser.
	  */
	public function getHtml() {
		// construct doctype
		switch($this->type) {
		case 'strict':
			$ret = '<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01//EN"' . PHP_EOL . 
			       '    "http://www.w3.org/TR/html4/strict.dtd">' . PHP_EOL; break;
		case 'loose':
			$ret = '<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN"' . PHP_EOL . 
				   '    "http://www.w3.org/TR/html4/loose.dtd">' . PHP_EOL; break;
		case 'xhtml1-frameset':
			$ret = '<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Frameset//EN"' . PHP_EOL . 
				   '    "http://www.w3.org/TR/xhtml1/DTD/xhtml1-frameset.dtd">' . PHP_EOL; break;
		case 'xhtml1-strict':
			$ret = '<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN"' . PHP_EOL .
				   '    "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">' . PHP_EOL;
			$ret .= '<html xmlns="http://www.w3.org/1999/xhtml">' . PHP_EOL; break;
		case 'xhtml1-transitional':
			$ret = '<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN"' . PHP_EOL . 
				   '    "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">' . PHP_EOL;
			$ret .= '<html xmlns="http://www.w3.org/1999/xhtml">' . PHP_EOL; break;
		case 'xhtml11':
			$ret = '<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.1//EN"' . PHP_EOL . 
				   '    "http://www.w3.org/TR/xhtml11/DTD/xhtml11.dtd">' . PHP_EOL;
			$ret .= '<html xmlns="http://www.w3.org/1999/xhtml">' . PHP_EOL; break;
		}
		
		// add head element
		$ret .= PHP_EOL . '<head>' . PHP_EOL;
		$ret .= '  <title>' . $this->title . '</title>' . PHP_EOL;
		if ($this->base != '')
			$ret .= '  <base href="' . $this->base . '" />' . PHP_EOL;
		$ret .= '  <meta http-equiv="content-type" content="text/html; charset=' . $this->enc . '" />' . PHP_EOL;
		
		// add external script files
		foreach ($this->js_includes as $inc)
			$ret .= '  <script type="text/javascript" src="' . $inc . '"></script>' . PHP_EOL;
			
		// add script blocks as CDATA sections
		foreach ($this->scripts as $script)
			$ret .= '  <script type="text/javascript">' . PHP_EOL . 
				    '    //<![CDATA[' . PHP_EOL . $script . PHP_EOL . '    //]]>' . PHP_EOL . '  </script>' . PHP_EOL;
		
		// add external style sheets 
		foreach ($this->css_includes as $inc)
			$ret .= '  <link rel="stylesheet" type="text/css" href="' . $inc . '" />' . PHP_EOL;
		
		// add strings
		foreach ($this->strings as $s) {
			$ret .= $s . PHP_EOL;
		}
		
		$ret .= '</head>' . PHP_EOL . PHP_EOL;
		
		$ret .= '<body' . ($this->onloadListener ? (' onload="' . $this->onloadListener . '"') : '') . '>' . PHP_EOL;
		
		return $ret;
	}
	public function __toString() {
		return $this->getHtml();
	}
	
	/**
	  * Adds a .css or .js file for inclusion.
	  *
	  * @param string $path The path to a .css or .js file to be included.
	  */
	public function addFile($path) {
		if (file_exists($path)) {
			// extract file extension
			$ext = substr($path, strrpos($path, '.'), strlen($path));
			if ($ext == '.css') {
				if (!in_array($path, $this->css_includes))
					$this->css_includes[] = $path;
			}
			elseif ($ext == '.js') {
				if (!in_array($path, $this->js_includes))
					$this->js_includes[] = $path;
			}
			else
				throw new Exception('Format of file: ' . $path . ' not recognized.');
		} else
			throw new Exception('File ' . $path . ' not found.');
	}
	
	/**
	  * Adds a JavaScript block that is directly written into the &lt;head&gt;.
	  *
	  * @param $script string A string containing the JavaScript block.
	  */
	public function addScriptBlock($script) {
		$this->scripts[] = $script;
	}
	
	/**
	  * Adds a .css or .js file for remote inclusion (via its URI).
	  *
	  * @param $uri string A string containing the URI of the file to be included.
	  * @param $type string Either 'css' or 'js' to denote the remote file's type.
	  */
	public function addRemoteFile($uri, $type) {
		if ($type == 'css')
			$this->css_includes[] = $uri;
		elseif ($type == 'js')
			$this->js_includes[] = $uri;
		else
			throw new Exception('Unkown remote file type: ' . $type . '.');
	}
	
	/**
	  * Adds an arbitrary string to the head. This way you can include
	  * meta tags or non-standard code like Internet Explorer specific comments
	  *
	  * @param string $string The string to be included &ndash; including its
	  *        surrounding tags.
	  */
	public function addString($string) {
		$this->strings[] = $string;
	}
	
	public function addOnloadListener($listener) {
		$this->onloadListener = $listener;
	}
}
?>
