<?php
/**
  * A basic class that provides OntoWiki's box functionality.
  *
  * @package html
  * @author Norman Heino
  * @version $Id: LayoutBox.php 887 2007-03-25 08:24:00Z nheino $
  */
class OntoWiki_Html_LayoutBox {
	
	// variable to hold the rendered html box content
	protected $content_string;
	protected $display_content;
	protected $_css_id = '';
	
	/**
	  * Constructs an OntoWiki css box.
	  *
	  * @param $title
	  * @param $css_id
	  * @param $content_css_name
	  */
	public function __construct($title, $css_id = '', $content_css_name = '', $content_visible = true) {
		
		$this->display_content = $content_visible;
		$this->css_id = css_id;
		
		$this->content_string  = '<div class="box"' . (($css_id != '') ? (' id="' . $css_id . '"') : '') . '>' . PHP_EOL;
		$this->content_string .= $this->getTitleHtml($title);
		$this->content_string .= '  <div class="boxcontent' . 
		                         ($content_visible ? '' : '" style="display:none') . 
								 (($content_css_name != '') ? (' name="' . $content_css_name) : '') . 
								 '">' . PHP_EOL;
	}
	
	/**
	  * Destructs the associated box object and unsets all abject variables.
	  */
	public function __destruct() {
		unset($this->content_string);
		unset($this->display_content);
	}
	
	/**
	  * Renders the box as a string which is then returned.
	  *
	  * @return string The box with its content as a rendered string.
	  */
	public function __toString() {
		// close css boxes
		$this->content_string .= PHP_EOL . '  </div> <!-- boxcontent -->' . PHP_EOL;
		$this->content_string .= '</div> <!-- box -->' . PHP_EOL . PHP_EOL;
		
		return $this->content_string;
	}
	
	/**
	  * Adds an arbitrary string to the box content.
	  *
	  * @param string $string
	  */
	public function addString($string) {
		$this->content_string .= $string;
	}
	
	/**
	  * Adds a label to the box content. A label is usually used to describe
	  * a list of elements.
	  *
	  * @param string $label A string containing the label text.
	  */
	public function addLabel($label) {
		$this->content_string .= '<b>' . $label . '</b>';
	}
	
	/**
	  * Adds a list to the box content. The list can be configured to be unordered
	  * (&lt;ul&gt;), ordered (&lt;ol&gt;), horizontal, vertical and with or without bullets.
	  *
	  * @param array $items An array of items to be listed.
	  * @param bool $unordered Denotes if the list is unordered (default) or ordered.
	  * @param bool $vertical Denotes if the list is vertical (default) or horizontal. Items
	  *        of a horizontal list are separated by vertical bars ("|").
	  * @param bool $with_bullet Denotes if an unordered, vertical list should be displayed
	  *        with bullets. This parameter has no effect on ordered or horizontal lists.
	  */ 
	public function addList($items, $unordered = true, $vertical = true, $with_bullet = false) {
		if (!empty($items)) {
			if ($unordered)
				$this->content_string .= '    <ul' . 
				($vertical ? 
					(!$with_bullet ? ' class="no_bullet">' : '>') : 
					' class="horizontal">') . PHP_EOL;
			else
				$this->content_string .= '    <ol>' . PHP_EOL;

			$array_size = count($items);
			for ($i = 0; $i < $array_size; ++$i) {
				$this->content_string .= '      <li' . 
				                         ($vertical ? '>' : ' class="horizontal">') . 
					                     $items[$i] . 
					                     (($i != $array_size-1 && !$vertical) ? '&nbsp;|' : '') . 
					                     '</li>' . PHP_EOL;
			}
			$this->content_string .= ($unordered ? '    </ul>' : '    </ol>') . PHP_EOL;
		}
	}
	
	/**
	  * Adds a horizontal line (&lt;hr /&gt;) to the box content.
	  */
	public function addLine($attrib = '') {
		$this->content_string .= '    <hr ' . $attrib . ' />' . PHP_EOL;
	}
	
	/**
	  * Adds an html form to the box content.
	  *
	  * @param string $action The form's action url. Often an action url is a script
	  *        that processes the form data.
	  * @param string $method The method by which the entered data should be submitted
	  *        to the action url. Can be either "get", "post" or "".
	  * @param array $items An array of form elements.
	  */
	public function addForm($action, $method, $items) {
		$this->content_string .= '    <form' . 
			(($method != '') ? (' method="' . $method . '"') : '') . 
			' action="' . $action . '">' . PHP_EOL;
		
		foreach ($items as $form_item) {
			$this->content_string .= '      ' . $form_item . PHP_EOL;
		}
		
		$this->content_string .= '    </form>' . PHP_EOL;
	}
	
	protected function getTitleHtml($title) {
		$ret = '  <div class="boxtitle">';
		if (is_array($title)) {
			$array_size = count($title);
			for ($i = 0; $i < $array_size; ++$i) {
				$ret .= $title[$i];
				if ($i != $array_size-1)
					$ret .= ' | ';
			}
		} else {
			$ret .= $title;
		}
		$ret .= '<a class="title_switch" onclick="toggleBoxContent(this);">' . 
				($this->display_content ? '–</a>' : '</a>');
		$ret .= '</div>' . PHP_EOL;
		
		return $ret;
	}
	
	protected function makeList($items) {
		// :TODO: make list out of items
	}
}
?>
